/* Aranha Parent process manager
 *
 * Copyright Daniel Silverstone <dsilvers@digital-scurf.org>
 * Copyright (C) 2004-2005 Pepperfish Ltd
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#include "config.h"
#include "configurator.h"
#include "lua.h"
#include "lauxlib.h"
#include "parentchild.h"
#include "fastcgi/lua_fastcgi.h"
#include "modules.h"

#include <stdlib.h>
#include <signal.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>

static void sig_child(int sig) 
{ 
  /* Completely non-interested in death of our kids */
  int temp;
  while( wait3( &temp, WNOHANG, NULL ) > 0 ) { }
}

void PrepareChild(int statecount)
{
  int i;
  signal( SIGCHLD, sig_child );
  for( i = 0; i < statecount; ++i ) {
    invoke_module_event( states[i], postfork );
  }
}

void ChildLoop(int statecount, int reqperstate)
{
  /* Work backward through the state array until
   * we have no more states left. then exit
   */
  
  while( statecount-- ) {
    lua_State *L = states[statecount];
    int reqsleft = reqperstate;
    while( reqsleft-- ) {
      if( AcceptFCGIRequest( L ) == 0 ) {
        /* Okay, we got a request... *RUN* with it */
        lua_getglobal( L, "__ARANHA_MAIN" );
        lua_call( L, 0, 0 );
        /* And terminate the FCGI request */
        TerminateFCGIRequest();
      }
    }
    /* Finally, close this state */
    invoke_module_event( L, closemodule );
    lua_close(L);
  }
  
  aranha_dlclose_everything();
  exit( 0 );
}
