## Copyright (C) 2004-2005 Pepperfish Ltd

## Permission is hereby granted, free of charge, to any person
## obtaining a copy of this software and associated documentation
## files (the "Software"), to deal in the Software without
## restriction, including without limitation the rights to use, copy,
## modify, merge, publish, distribute, sublicense, and/or sell copies
## of the Software, and to permit persons to whom the Software is
## furnished to do so, subject to the following conditions:

## The above copyright notice and this permission notice shall be
## included in all copies or substantial portions of the Software.

## THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
## EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
## MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
## NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
## HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
## WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
## OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
## DEALINGS IN THE SOFTWARE.

# Important include path for lua
AM_CFLAGS += -I$(top_srcdir)/lua/src -DLUA_CONSTRUCTORS -DLUA_CTRARGS=3 -I$(top_srcdir)/lua/dynasm -std=gnu99 -Wno-comment -Ilua/src
AM_LDFLAGS += -lm -ldl

# List the files for building LUA and LUAC

lua_sources := lua/src/lapi.c lua/src/lapi.h lua/src/lcode.c		\
lua/src/lcode.h lua/src/ldebug.c lua/src/ldebug.h lua/src/ldo.c		\
lua/src/ldo.h lua/src/ldump.c lua/src/lfunc.c lua/src/lfunc.h		\
lua/src/lgc.c lua/src/lgc.h lua/src/llex.c lua/src/llex.h		\
lua/src/llimits.h lua/src/lmem.c lua/src/lmem.h lua/src/lobject.c	\
lua/src/lobject.h lua/src/lopcodes.c lua/src/lopcodes.h			\
lua/src/lparser.c lua/src/lparser.h lua/src/lstate.c lua/src/lstate.h	\
lua/src/lstring.c lua/src/lstring.h lua/src/ltable.c lua/src/ltable.h	\
lua/src/ltm.c lua/src/ltm.h lua/src/lundump.c lua/src/lundump.h		\
lua/src/lvm.c lua/src/lvm.h lua/src/lzio.c lua/src/lzio.h		\
lua/src/lua.h lua/src/lauxlib.h lua/src/luaconf.h lua/src/lauxlib.c	\
lua/src/lbaselib.c lua/src/ldblib.c lua/src/linit.c lua/src/liolib.c	\
lua/src/loadlib.c lua/src/loslib.c lua/src/lstrlib.c lua/src/ltablib.c	\
lua/src/lualib.h lua/src/lcutils.c lua/src/lcutils.h lua/src/lcoco.c    \
lua/src/lcoco.h lua/src/ljit_backend.c lua/src/ljit_core.c              \
lua/src/ljit_dasm.c lua/src/ljit_dasm.h lua/src/ljit.h                  \
lua/src/ljit_hints.h lua/src/ljitlib.c lua/src/ljit_mem.c               \
lua/src/luajit.h
if JIT_BACKEND_X86
lua_sources += lua/dynasm/dasm_proto.h lua/dynasm/dasm_x86.h
endif

noinst_LIBRARIES := lua/src/liblmathlib.a
lua_src_liblmathlib_a_SOURCES := lua/src/lmathlib.c
lua_src_liblmathlib_a_CFLAGS := $(C99FLAGS)

LUAC_SRC := lua/src/luac.c lua/src/print.c $(lua_sources)
LUAC_BIN := lua/bin/luac$(EXEEXT)

lua_bin_luac_SOURCES := $(LUAC_SRC)
lua_bin_luac_LDADD := lua/src/liblmathlib.a

BIN2C_SRC := lua/etc/bin2c.c
BIN2C_BIN := lua/bin/bin2c$(EXEEXT)

lua_bin_bin2c_SOURCES := $(BIN2C_SRC)

STARTUP_OBJ := luastartup.luac
STARTUP_INC := luastartup.inc

STARTUP_SRC := lua/startup/00-Prepare \
               lua/startup/01-Types \
               lua/startup/05-Functions \
	       lua/startup/10-Classes \
	       lua/startup/15-Misc \
	       lua/startup/20-Docs \
	       lua/startup/30-Require \
	       lua/startup/99-Cleanup

EXTRA_DIST += $(STARTUP_SRC) lua/src/ljit_x86.dash lua/src/ljit_x86.dasc \
lua/src/ljit_x86_inline.dash lua/dynasm/dynasm.lua lua/dynasm/dasm_x86.lua

$(STARTUP_OBJ): $(LUAC_BIN)

$(STARTUP_OBJ): $(STARTUP_SRC)
	$(LUAC_BIN) -o $@ $(STARTUP_SRC:%=$(srcdir)/%)

$(STARTUP_INC): $(BIN2C_BIN)

$(STARTUP_INC): $(STARTUP_OBJ)
	$(BIN2C_BIN) $(STARTUP_OBJ) > $(STARTUP_INC)

LUA_SRC := lua/src/lua.c $(lua_sources)

LUA_BIN := lua/bin/lua$(EXEEXT)

lua/src/lua.$(OBJEXT): $(STARTUP_INC)

lua_bin_lua_SOURCES := $(LUA_SRC)
lua_bin_lua_LDADD := lua/src/liblmathlib.a

if JIT_ENABLED
PRELUA_BIN := lua/bin/prelua
lua_bin_prelua_SOURCES := $(LUA_SRC)
lua_bin_prelua_LDADD := lua/src/liblmathlib.a
lua_bin_prelua_CFLAGS := -DDISABLE_JIT -DDISABLE_ARANHA_STARTUP $(AM_CFLAGS)
endif

noinst_PROGRAMS += $(BIN2C_BIN) $(LUAC_BIN) $(LUA_BIN) $(PRELUA_BIN)

# The dasm magic happens here

if JIT_BACKEND_X86
lua/src/.deps/ljit_backend.Po lua/src/ljit_backend.$(OBJEXT): lua/src/ljit_x86.h
lua/src/ljit_x86.h: lua/src/ljit_x86.dasc lua/src/ljit_x86.dash lua/bin/prelua
	lua/bin/prelua $(srcdir)/lua/dynasm/dynasm.lua -I $(srcdir)/lua/src -o lua/src/ljit_x86.h $(srcdir)/lua/src/ljit_x86.dasc
endif

## The tests...

LUA_TESTS = lua/test/000_compilercoverage lua/test/001_classsystem	\
lua/test/002_divertchecks lua/test/003_stringchecks \
lua/test/004_extramethods

EXTRA_DIST += $(LUA_TESTS)
TESTS += $(LUA_TESTS)

## Cleanup

clean-local: lua-clean-local

lua-clean-local:
	rm -f $(STARTUP_OBJ) $(STARTUP_INC)
	rm -f lua-test-compilercoverage-ok
	rm -f lua-test-classchecks-ok
	rm -f lua-test-divertchecks-ok
	rm -f lua-test-stringchecks-ok
	rm -f lua-test-extramethodchecks-ok
	rm -f lua/src/ljit_x86.h

pkginclude_HEADERS += lua/src/lua.h lua/src/lauxlib.h lua/src/luaconf.h
