# jit.m4 -- configure macros for LuaJIT integration in aranha
#
# Copyright © 2006 Daniel Silverstone <dsilvers@digital-scurf.org>
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR
# ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


# ARANHA_PROBE_JIT
# ----------------------
# Check whether or not the JIT is likely to be usable and set some variables
# We do this by examining MACHTYPE and considering from there
AC_DEFUN([ARANHA_PROBE_JIT],
[AC_MSG_CHECKING([if target could support LuaJIT])
case $target_cpu in
  i?86)
    ARANHA_JIT_CPU_OK=y
  ;;
  *)
    ARANHA_JIT_CPU_OK=n
  ;;
esac
case $target_os in
  linux-gnu)
    ARANHA_JIT_OS_OK=y
  ;;
  *)
    ARANHA_JIT_OS_OK=n
  ;;
esac

if test "$ARANHA_JIT_CPU_OK$ARANHA_JIT_OS_OK" = "yy"; then
  ARANHA_JIT_POSSIBLE=y
  ARANHA_JIT_MSG="CPU/OS combination plausibly suported"
else
  ARANHA_JIT_POSSIBLE=n
  ARANHA_JIT_MSG="CPU/OS combination unlikely to be supported"
fi
AC_MSG_RESULT([$ARANHA_JIT_MSG])
])# ARANHA_PROBE_JIT

# Define a --enable-jit argument
# If you pass --enable-jit=force then it will do its best to force the JIT on
AC_DEFUN([ARANHA_JIT_ARG],
[AC_ARG_ENABLE(jit,AS_HELP_STRING([--enable-jit],
[use LuaJIT. Default is system dependent]),[
ARANHA_JIT_USE_VALGRIND=n
case x$enableval in
 xyes)
  ARANHA_JIT_USER_ENABLE=y
 ;;
 xvalgrind)
  ARANHA_JIT_USER_ENABLE=y
  ARANHA_JIT_USE_VALGRIND=y
 ;;
 xforce)
  ARANHA_JIT_USER_ENABLE=f
 ;;
 *)
  ARANHA_JIT_USER_ENABLE=n
esac
],[
ARANHA_JIT_USER_ENABLE=$ARANHA_JIT_POSSIBLE
ARANHA_JIT_USE_VALGRIND=n
])
])# ARANHA_JIT_ARG

# When called, resolves the decisions about JITness and sets ARANHA_JIT_ENABLED to yes or no
AC_DEFUN([ARANHA_JIT_RESOLVE],
[
case $ARANHA_JIT_POSSIBLE$ARANHA_JIT_USER_ENABLE in
 yy) ARANHA_JIT_ENABLED=yes ;;
 yf) ARANHA_JIT_ENABLED=yes ;;
 nn) ARANHA_JIT_ENABLED=no ;;
 yn) AC_MSG_WARN([LuaJIT disabled at user's request])
     ARANHA_JIT_ENABLED=no ;;
 ny) AC_MSG_WARN([LuaJIT is unlikely to work. Ignoring user request.])
     AC_MSG_WARN([Pass --enable-jit=force to attempt to force the issue.])
     ARANHA_JIT_ENABLED=no ;;
 nf) AC_MSG_WARN([LuaJIT is unlikely to work, but forcing it to be enabled anyway.])
     ARANHA_JIT_ENABLED=yes ;;
 *) AC_MSG_WARN([Unknown combination of possible/enable for LuaJIT. Disabling JIT])
    ARANHA_JIT_ENABLED=no ;;
esac
AC_MSG_CHECKING([whether or not jit is to be enabled])
AC_MSG_RESULT([$ARANHA_JIT_ENABLED])
])# ARANHA_JIT_RESOLVE

# When invoked, updates CFLAGS etc with the requisite 
AC_DEFUN([ARANHA_JIT_FLAGS],
[AC_MSG_CHECKING([for flags appropriate to jit choice])
if test "x$ARANHA_JIT_ENABLED" = "xno"; then
  JIT_CFLAGS="-DDISABLE_JIT"
  JIT_LDFLAGS=""
  JIT_BACKEND=""
else
  JIT_CFLAGS=""
  JIT_LDFLAGS=""
  case $target_os in
  linux-gnu)
    JIT_CFLAGS="$JIT_CFLAGS -DLUA_USE_LINUX"
    JIT_LDFLAGS="$JIT_LDFLAGS -Wl,-E"
    ;;
  esac
  case $target_cpu in
   i?86)
     JIT_BACKEND=x86
  esac
fi
if test "x$ARANHA_JIT_USE_VALGRIND" = "xy"; then
  JIT_CFLAGS="$JIT_CFLAGS -DUSE_VALGRIND"
fi
AC_MSG_RESULT([cflags=$JIT_CFLAGS, ldflags=$JIT_LDFLAGS, jit_backend=$JIT_BACKEND])
])# ARANHA_JIT_FLAGS

# When invoked, performs AM_CONDITIONAL etc
AC_DEFUN([ARANHA_JIT_SUBST],
[AC_SUBST(JIT_CFLAGS)
AC_SUBST(JIT_LDFLAGS)
AC_SUBST(JIT_BACKEND)
AM_CONDITIONAL(JIT_ENABLED, [test "x$ARANHA_JIT_ENABLED" = "xyes"])
AM_CONDITIONAL(JIT_BACKEND_X86, [test "x$JIT_BACKEND" = "xx86"])
])# ARANHA_JIT_SUBST
